/*****************************************************************************
 *****************************************************************************
 Copyright (c) 2000-2001, AEP Ltd. 

 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.

 3. Neither the name of AEP Ltd. nor the names of its contributors 
 may be used to endorse or promote products derived from this software 
 without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

 *****************************************************************************
****************************************************************************/

/**********************************************************************
*                                                                     *
* AEP Ltd.                                                            *
*                                                                     *
* This software is supplied under the terms of the license included   *
* above.  All use of this software must be in accordance with the     *
* terms of that license.                                              *
*                                                                     *
**********************************************************************/

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include "aeptrace_config.h"
#include "aeptrace.h"

/*************************************************************************************
   aeptrace_config.c 
	
   This file contains code which configures tracing parameters as set
   by the user in aeplog.conf
 
*************************************************************************************/



extern TRACE_CONFIG_ITEM TRACE_OPTIONS[NUM_TRACE_CONFIG_ITEMS];
extern AEP_U32 setTraceRV;

/******************************************************************************************************
	void setTraceConfigValues()	

	This function reads the tracing options required by the user from aeplog.conf and populates 
	the global array of structures with these values 
******************************************************************************************************/
void setTraceConfigValues()
{	

	FILE *configFile;
	char line[MAX_LINE_LENGTH];
	int lineIndex = 0;
	int tempIndex;
	int lineLength;
	char *onString = "ON";
	char *offString = "OFF";
	char *quotation = "\"";
	char valueString[MAX_FILE_LENGTH];
	TRACE_CONFIG_ITEM temp;
	AEP_U32 number;
	int valueFound = 0;
	int value;
	AEP_U32 valueLength;
	int maxLineLength = MAX_LINE_LENGTH;
	
	/* Initially no errors */
	setTraceRV = AEP_R_OK;
	
	/* Open configuration file */
	configFile = fopen(TRACE_CONFIG_FILE, "r");
	
	if(configFile == NULL)
	{
		setTraceRV =  AEP_R_FILE_OPEN_ERROR;
		perror("\nERROR - Unable to Open Trace Config File");
		return;
	}	

	/* Parse each line in the file */
	while(fgets(line, maxLineLength, configFile) != NULL)
	{
		lineIndex = 0;
		valueFound = 0;
		temp.Status = OFF;
		
		/* Get the line length */
		lineLength = strlen(line);
		
		/* Parse Line*/
		/* Get rid of any leading blanks */
		while(line[lineIndex] == ' ')
			lineIndex++;

		/* Ignore lines starting with ';' or ']' - these are comment lines */
		if( (lineIndex < lineLength) && (line[lineIndex] != ';') && (line[lineIndex]!= '[') )
		{
			/* Get Item and Status */
			/* If line begins with `i` this is the item line and marks the start of a configuration record */
			if((line[lineIndex] == 'i') || (line[lineIndex] == 'I'))
			{		
				/* Line is of form 'item = 4'
				  Skip over `item` string */
				lineIndex += 4;
				
				/* Skip over any blanks and the equals sign */
				while((line[lineIndex] == ' ') || (line[lineIndex] == '='))
					lineIndex++;
		  
				/* Get Number denoting the configuration item*/
				/* Sscanf is the string scan function */
				sscanf(&line[lineIndex], "%d", &number);
			
				/* Must get the value ie ON/OFF for this item next so skip over any non-value lines*/
				do
				{
					valueLength = 0;
					lineIndex = 0;
					valueLength = 0;
					/* Get the next line from the file */
					if(fgets(line, 100, configFile) == NULL)
					{
						setTraceRV = AEP_R_EOF;
					}	
					else
					{
						/* Get the length of the line */
						lineLength = strlen(line);
						
						/* Skip blanks */
						while(line[lineIndex] == ' ')
						lineIndex++;
		  
						/* Ignore comment lines or lines */
						if( (lineIndex < lineLength) && (line[lineIndex] != ';') && (line[lineIndex] != '[') )
						{
				     		if ((line[lineIndex] == 'v') || (line[lineIndex] ==	'V'))
							{
								/* Line is of form value = ON or value = "string" */
								/* Skip over 'value'*/
								lineIndex += 5;

								/* Skip over blanks and '=' */
								while((line[lineIndex] == ' ') ||(line[lineIndex] == '='))
									lineIndex++;
								/* if value = ON */
								if(strncmp(&line[lineIndex], onString, 2) == 0)
				 					value = ON;
								/* else if value = OFF */
								else if (strncmp(&line[lineIndex], offString,3)==0)
									value = OFF;
 								/* else if value is a string */
								else if (strncmp(&line[lineIndex],quotation,1) == 0)
								{
									lineIndex++; 
									tempIndex = lineIndex;
									/* Get length of string by counting to end quotation mark */
									while((strncmp(&line[tempIndex],quotation,1 != 0))&&(tempIndex < MAX_LINE_LENGTH))
									{
										valueLength++;
										tempIndex++;
									}
									/* Copy this string into temp variable */
 									strncpy(valueString, &line[lineIndex], valueLength);						
								}
								/* By default set value to OFF */
								else
									value = OFF;
								valueFound = 1;
							}/* end if value line */	
						}/*end if */
					}/* end else */
				}while((valueFound == 0) &&(setTraceRV == AEP_R_OK));	

	   		    /* If everything is ok, set the configuration item */
				if(setTraceRV == AEP_R_OK)
				{
					setConfigItem(number, value, valueString, valueLength);
				}	
			}/* end if line starting with 'item' */

		}/* end if not a comment line */
	
	}/* end while - file parsed */

	return;
}
/******************************************************************************************************/
