/*****************************************************************************
 *****************************************************************************
 Copyright (c) 2000-2001, AEP Ltd. 

 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.

 3. Neither the name of AEP Ltd. nor the names of its contributors 
 may be used to endorse or promote products derived from this software 
 without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

 *****************************************************************************
****************************************************************************/

/**********************************************************************
*                                                                     *
* AEP Ltd.                                                            *
*                                                                     *
* This software is supplied under the terms of the license included   *
* above.  All use of this software must be in accordance with the     *
* terms of that license.                                              *
*                                                                     *
**********************************************************************/
#ifndef AEPTRACECONFIG_H
#define AEPTRACECONFIG_H

#include "aep_i_defs.h"
#include "aep_i_types.h"
#include "aep_api.h"

#define ON 1
#define OFF 0

#define NUM_TRACE_CONFIG_ITEMS			24
#define MAX_FILE_LENGTH					100
#define MAX_LINE_LENGTH					100

typedef struct TRACE_CONFIG_ITEM
{
	AEP_U32 item;	
	AEP_U32 Status; /*Either on or off */
	AEP_CHAR name[MAX_FILE_LENGTH]; /* If value is string */
	AEP_U32 length;	/* length of name */

}TRACE_CONFIG_ITEM;
/*--------------------------------------------------------------------------*/
/*  Trace Options - possible levels of tracing available					*/
/*--------------------------------------------------------------------------*/

/* Tracing On/Off */
#define AEP_DAEMON_TRACE_STATUS		10

#define AEP_API_TRACE_STATUS		11

/*Write to Console*/
#define STREAM_OUTPUT_ENABLED		12

/*Write to File*/
#define FILE_OUTPUT_ENABLED			13

/*Prefix each entry with timestamp*/
#define TIME_STAMP_ENABLED			14

/*Record Process ID for each log entry */
#define SHOW_PROCESS_ID				15

/*Record Thread ID for each log entry*/
#define SHOW_THREAD_ID				16

#define TRACE_ERRORS				17
#define TRACE_DEBUG					18
#define TRACE_WARNINGS				19
#define TRACE_SUMMARY				20

#define API_LOG_DIR					21
#define DAEMON_LOG_DIR				22
#define DRIVER_LOG_DIR				23

#ifdef AEP_GENERIC
#define TRACE_CONFIG_FILE			"/etc/aeplog.conf"
#elif defined AEP_Win32
#define TRACE_CONFIG_FILE			"c:\\aeplog.conf"
#endif

#define TIDWidth					20
#define PIDWidth					20
#define FNNameWidth					25
#define RelTimeWidth				25
#define RVWidth						20

/* Function Prototypes */
void setTraceConfigValues();
int initTraceConfigItems();

#endif
