/*****************************************************************************
 *****************************************************************************
 Copyright (c) 2000-2001, AEP Ltd. 

 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.

 3. Neither the name of AEP Ltd. nor the names of its contributors 
 may be used to endorse or promote products derived from this software 
 without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

 *****************************************************************************
****************************************************************************/

/**********************************************************************
*                                                                     *
* AEP Ltd.                                                            *
*                                                                     *
* This software is supplied under the terms of the license included   *
* above.  All use of this software must be in accordance with the     *
* terms of that license.                                              *
*                                                                     *
**********************************************************************/

/* **************************************************************/
/*								*/
/* Filename:	init.c						*/
/* Description:	This file contains the implementation of the 	*/
/*		API init/uninit functions. 		 	*/
/*								*/
/* Copyright:	Accelerated Encryption Processing Ltd., 2000	*/
/* Author:	Diarmuid O'Neill				*/
/* Product:	AEP1000						*/
/* Date:	13/10/99					*/
/*								*/
/****************************************************************/



#ifdef AEP_Win32
# include <windows.h>
#elif AEP_GENERIC
# include <dlfcn.h>
# include <unistd.h>

#endif /* !AEP_Win32 */

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#include "aep_i_defs.h"
#include "aep_i_types.h"

#include "aep_api.h"
#include "aep_api_globals.h"
#include "mutex.h"
#include "init.h"
#include "aepmalloc.h"
#include "aependian.h"
#include "connection.h"
#include "SrvInternals.h"
#include "aeptrace.h"

extern AEP_I_CONNECTION_DATA connection_array[MAX_CONNECTIONS];
extern AEP_U32 AEP_I_tran_id_counter;

/*Structure contains pointers to the various bignum manipulation functions.*/
AEP_I_EXT_FUNCTION_LIST AEP_I_ext_functions;

/* This is the global flags variable that indicates if the api was initialised*/
AEP_U32 AEP_I_global_flags;
AEP_U64	AEP_I_app_ID = 0;

/*This synchronisation variables are used to marshall shared data.  ie the global flags
	and the hash table*/
pthread_mutex_t AEPAPI_flags_mutex = PTHREAD_MUTEX_INITIALIZER;

/*	AEP_Initialize:
	This function initialises the api by setting the AEP_I_global_flags variable
	and by setting up the connections hash table

	Inputs	:	Pointer to initargs  {pInitArgs}
	Outputs	:	None
*/

AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_Initialize(
 AEP_VOID_PTR	pInitArgs
)
{
	AEP_RV rv = AEP_R_OK;
#ifndef VSA_API
	AEP_CONNECTION_HNDL hConnection;
#endif
	AEP_U32 i;
	AEP_U16 tmp;
	
	AEP_TRACE(API, TRACE_DEBUG, FN_IN, "AEP_Initialize", AEP_R_OK, "Entered....");

/* Protect the global flags variable */ 
	_LOCK(&AEPAPI_flags_mutex);

  /* Check to see if the API is already initialised */
	if ((AEP_I_global_flags & AEP_IGF_INITIALIZED)) 
    {
		rv = AEP_R_AEPAPI_ALREADY_INITIALIZED;
		AEP_TRACE(API, TRACE_ERRORS, FN_NONE, "AEP_Initialize", rv, "API Already Initialised");
		_UNLOCK(&AEPAPI_flags_mutex);
		return rv;
    }

  /*Set the default BigNum manipulation functions*/
	AEP_I_ext_functions.AEP_I_GetBigNumSize		= &GetBigNumSize;
	AEP_I_ext_functions.AEP_I_MakeAEPBigNum		= &MakeAEPBigNum;
	AEP_I_ext_functions.AEP_I_ConvertAEPBigNum  = &ConvertAEPBigNum;

/*Mark each entry in the connection array as free*/
	for(i=0;i<MAX_CONNECTIONS;i++)
		connection_array[i].status = 0;
      
  /* Initalization successfull*/
	AEP_I_global_flags |= AEP_IGF_INITIALIZED;

/*Finished with the global flags variable */
	_UNLOCK(&AEPAPI_flags_mutex);

#ifndef VSA_API
  /*Initialize the counter in the tranid generator with a rnd number*/
        rv = AEP_OpenConnection(&hConnection);
	if(rv != AEP_R_OK)
	{
		AEP_TRACE(API, TRACE_ERRORS, FN_NONE, "AEP_Initialize", rv, "Could not connect.. failed");
		AEP_I_global_flags &= ~AEP_IGF_INITIALIZED;
		return rv;
	}

        rv = AEP_GenRandom(hConnection, 2, 2, (void*)&tmp, NULL_PTR);
	if(rv != AEP_R_OK)
	{
		AEP_TRACE(API, TRACE_ERRORS, FN_NONE, "AEP_Initialize", rv, "Could not generate Random Number... failed");
		AEP_I_global_flags &= ~AEP_IGF_INITIALIZED;
		return rv;
	}

        rv = AEP_CloseConnection(hConnection);
	if(rv != AEP_R_OK)
	{
		AEP_TRACE(API, TRACE_ERRORS, FN_NONE, "AEP_Initialize", rv, "Could not close Connection...failed");
		AEP_I_global_flags &= ~AEP_IGF_INITIALIZED;
		return rv;
	}
#endif
  /*Assign the rnd number to the counter that doles out the tran ids*/
        AEP_I_tran_id_counter = (AEP_U32) tmp;

  /*Must ensure that this value is not 0 which indicates Invalid tran id*/
	if(AEP_I_tran_id_counter == 0)
		AEP_I_tran_id_counter++;

	AEP_TRACE(API, TRACE_DEBUG, FN_OUT, "AEP_Initialize", AEP_R_OK, "...Leaving");
	return AEP_R_OK;
}
/* End of AEP_Initialize */


/* AEP_Finalize
	This function indicates that the application programmer is finished using the
	AEP API.  The function unsets the global flags variable and destroys the connection
	hash table
		
	Inputs	:	None
	Outputs	:	None
*/
AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_Finalize(
)
{
	AEP_RV rv = AEP_R_OK;
	AEP_TRACE(API, TRACE_DEBUG, FN_IN, "AEP_Finalize", AEP_R_OK, "Entered...." );

/* Protect the global flags variable */ 
	_LOCK(&AEPAPI_flags_mutex);

  /* check to see if the API is initialised */
	if (!(AEP_I_global_flags & AEP_IGF_INITIALIZED))
	{
		AEP_TRACE(API, TRACE_ERRORS, FN_NONE, "AEP_Finalize", AEP_R_AEPAPI_NOT_INITIALIZED, "Error : API not initialised");
		_UNLOCK(&AEPAPI_flags_mutex);
		return AEP_R_AEPAPI_NOT_INITIALIZED;
	}

  /*We are no longer initialised*/
	AEP_I_global_flags ^= AEP_IGF_INITIALIZED;

/*Finished with the global flags variable */
	_UNLOCK(&AEPAPI_flags_mutex);
	
	AEP_TRACE(API, TRACE_DEBUG, FN_OUT, "AEP_Finalize", AEP_R_OK, "....Leaving");

	return rv;
}
/*End AEP_Finalize*/


/*	AEP_Auth_App:
	This function authenticates an application with the target.

	Inputs	:	Connection Handle  {hConnection}
			The App's text name {bbAppName}

	Outputs	:NONE	
	*/
AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_Auth_App(
	AEP_CONNECTION_HNDL		hConnection,
	AEP_BYTEBLOCK			bbAppName
	)
{
	AEP_BYTEBLOCK	commandblock, replyblock;
	AEP_U32		datalen, index, tran_id;
	AEP_MSG_HDR_PTR	message_hdr;
 
	AEP_RV			rv = AEP_R_OK;


	AEP_TRACE(API, TRACE_DEBUG, FN_IN, "AEP_Auth_App", AEP_R_OK, "Entered...");

/* Protect the global flags variable */ 
	_LOCK(&AEPAPI_flags_mutex);

  /* make sure we are initialized */
	if (!(AEP_I_global_flags & AEP_IGF_INITIALIZED))
	{
		AEP_TRACE(API, TRACE_ERRORS, FN_NONE, "AEP_Auth_App", AEP_R_AEPAPI_NOT_INITIALIZED, "Error : API Already initialised");
		_UNLOCK(&AEPAPI_flags_mutex);
		return AEP_R_AEPAPI_NOT_INITIALIZED;
	}

  /* Get the Tran ID to assign to this request*/
	rv = AEP_I_NewTranID(&tran_id);
	if(rv != AEP_R_OK)
	{
		AEP_TRACE(API, TRACE_ERRORS, FN_NONE, "AEP_Auth_App", rv, "Error : failed on connection %l", hConnection);			
		_UNLOCK(&AEPAPI_flags_mutex);
		return rv;
	}

/*Finished with the flags */
	_UNLOCK(&AEPAPI_flags_mutex);

  /*Calculate the size of the message data*/
	datalen = bbAppName.Len + sizeof(AEP_U32);

  /*How big is this byteblock going to be?*/
	commandblock.Len = HDR_SIZE + datalen;

  /*Allocate a buffer big enough to hold the message (Header + Data)*/
	commandblock.ptr=(AEP_U8_PTR)AEP_malloc(commandblock.Len);
	if(commandblock.ptr == NULL_PTR)
	{
		AEP_TRACE(API, TRACE_ERRORS, FN_NONE, "AEP_Auth_App", AEP_R_HOST_MEMORY, "Error : Host Memory Allocation Failure");
		return AEP_R_HOST_MEMORY;
	}

  /*Clear the header part of the commandblock*/
	memset((void*)commandblock.ptr,0,HDR_SIZE);
 
  /*We make this assignemt so we can refer to our message hdr using the msghdr struct*/
	message_hdr = (AEP_MSG_HDR_PTR)commandblock.ptr;

  /*Copy the command identifier, the data length and the status*/
	message_hdr->command = AEP_C_AUTH_APP;

  /*Copy the transaction handle for this request*/
	message_hdr->trans_hndl = (AEP_U64) getpid() << 32 | tran_id;

  /*This command is for the accelerator device and goes to the SA engine*/
	message_hdr->flags = AEP_F_CTXCMD | AEP_F_SA;

  /*Set the message length field*/
	message_hdr->datalen = datalen;

  /*Copy the command's data into the message*/
  /*Set the index to the start of the message data*/
	index = HDR_DATA_OFFSET;

  /*Copy the length of the plain text block and adjust the endianism*/	
	*((AEP_U32_PTR)(commandblock.ptr + index)) = (AEP_U32)(bbAppName.Len);
	BEndToLEnd(commandblock.ptr + index, 1);
	index+=sizeof(AEP_U32);

  /*Copy the plain text block*/
	memcpy(commandblock.ptr + index, bbAppName.ptr, bbAppName.Len);
	
 	rv = AEP_I_Transact(hConnection, &commandblock, &replyblock);

	if(rv != AEP_R_OK)
	{
		AEP_TRACE(API, TRACE_ERRORS, FN_NONE, "AEP_Auth_App", rv, "Error : failed on connection %l", hConnection);

	  /*Free the data block that was used to hold the reply*/
		AEP_free(replyblock.ptr);
	  /*Free the data block that was used to hold the message*/
		AEP_free(commandblock.ptr);
		return rv;
	}

  /*Set the app id value equal to the context id returned in the reply*/
	AEP_I_app_ID = *((AEP_U64*)replyblock.ptr);
	
  /*Free the data block that was used to hold the reply*/
	AEP_free(replyblock.ptr);

  /*Free the data block that was used to hold the message*/
	AEP_free(commandblock.ptr);
	
	AEP_TRACE(API, TRACE_DEBUG, FN_OUT, "AEP_Auth_App", AEP_R_OK, "...Leaving");

	return rv; 
}

/*	AEP_UnAuth_App:
	This function marksan application's app id as invalid on the target.

	Inputs	:	Connection Handle  {hConnection}

	Outputs	:	None
	
	*/
AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_UnAuth_App(
	AEP_CONNECTION_HNDL		hConnection
	)
{
	AEP_BYTEBLOCK	commandblock;
	AEP_MSG_HDR_PTR	message_hdr;
 
	AEP_RV			rv = AEP_R_OK;

	AEP_TRACE(API, TRACE_DEBUG, FN_IN, "AEP_UnAuth_App", AEP_R_OK, "Entered...");

/* Protect the global flags variable */ 
	_LOCK(&AEPAPI_flags_mutex);

  /* make sure we are initialized */
	if (!(AEP_I_global_flags & AEP_IGF_INITIALIZED))
	{
		AEP_TRACE(API, TRACE_ERRORS, FN_NONE, "AEP_UnAuth_App", AEP_R_AEPAPI_NOT_INITIALIZED, "Error : API Already Initialised");
		_UNLOCK(&AEPAPI_flags_mutex);
		return AEP_R_AEPAPI_NOT_INITIALIZED;
	}

/*Finished with the flags */
	_UNLOCK(&AEPAPI_flags_mutex);

  /*How big is this byteblock going to be?*/
	commandblock.Len = HDR_SIZE;

  /*Allocate a buffer big enough to hold the message (Header + Data)*/
	commandblock.ptr=(AEP_U8_PTR)AEP_malloc(commandblock.Len);
	if(commandblock.ptr == NULL_PTR)
	{
		AEP_TRACE(API, TRACE_ERRORS, FN_NONE, "AEP_UnAuth_App", AEP_R_HOST_MEMORY, "Error : Host memory Allocation Failure");
		return AEP_R_HOST_MEMORY;
	}

  /*Clear the header part of the commandblock*/
	memset((void*)commandblock.ptr,0,HDR_SIZE);
 
  /*We make this assignemt so we can refer to our message hdr using the msghdr struct*/
	message_hdr = (AEP_MSG_HDR_PTR)commandblock.ptr;

  /*Copy the command identifier, the data length and the status*/
	message_hdr->command = AEP_C_UNAUTH_APP;

  /*Copy the transaction handle for this request*/
	message_hdr->trans_hndl = (AEP_U64) getpid() << 32;

  /*This command is for the accelerator device and goes to the SA engine*/
	message_hdr->flags = AEP_F_CTXCMD | AEP_F_SA;

  /*Set the message length field*/
	message_hdr->datalen = 0;

 	rv = AEP_I_Submit(hConnection, &commandblock);

	if(rv != AEP_R_OK)
	{
		AEP_TRACE(API, TRACE_ERRORS, FN_NONE, "AEP_UnAuth_App", rv, "Error : failed on connection %l", hConnection);

	  /*Free the data block that was used to hold the message*/
		AEP_free(commandblock.ptr);
		return rv;
	}

  /*Set the app id value equal to the context id returned in the reply*/
	AEP_I_app_ID = 0;
	
 /*Free the data block that was used to hold the message*/
	AEP_free(commandblock.ptr);
	
	AEP_TRACE(API, TRACE_DEBUG, FN_OUT, "AEP_UnAuth_App", AEP_R_OK, "...Leaving");

	return rv; 
}

/* AEP_SetBNCallBacks
	This function provides the addresses of the callback functions to be used when
	dealing with Bignums.
		
	Inputs	:	Pointer to GetBNSize function
	Outputs	:	None
*/
AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_SetBNCallBacks(
 AEP_RV		(*GetBigNumSizeFunc)(),
 AEP_RV		(*MakeAEPBigNumFunc)(),
 AEP_RV		(*ConvertAEPBigNumFunc)()
)
{
	AEP_RV rv = AEP_R_OK;

	AEP_TRACE(API, TRACE_DEBUG, FN_IN, "AEP_SetBNCallBacks", AEP_R_OK, "Entered.... ");

	AEP_I_ext_functions.AEP_I_GetBigNumSize = GetBigNumSizeFunc;
	
	AEP_I_ext_functions.AEP_I_MakeAEPBigNum = MakeAEPBigNumFunc;
	
	AEP_I_ext_functions.AEP_I_ConvertAEPBigNum = ConvertAEPBigNumFunc;

	AEP_TRACE(API, TRACE_DEBUG, FN_OUT, "AEP_SetBNCallBacks", AEP_R_OK, "...Leaving");


	return rv;
}
/*End AEP_Finalize*/

static AEP_RV GetBigNumSize(
	AEP_VOID_PTR	ArbBigNum, 
	AEP_U32*		BigNumSize
)
{
	printf("GetBigNumSize : Must provide callback functions! \n");
	exit(1);
	return 1;
}

static AEP_RV MakeAEPBigNum(
	AEP_VOID_PTR	ArbBigNum, 
	AEP_U32		BigNumSize, 
	unsigned char*	AEP_BigNum
)
{
	printf("MakeAEPBigNum : Must provide callback functions! \n");
	exit(1);
	return 1;
}
 
static AEP_RV ConvertAEPBigNum(
	AEP_VOID_PTR	ArbBigNum, 
	AEP_U32		BigNumSize, 
	unsigned char*	AEP_BigNum
)
{
	printf("ConverAEPBigNum : Must provide callback functions! \n");
	exit(1);
	return 1;
}
