/*****************************************************************************
 *****************************************************************************
 Copyright (c) 2000-2001, AEP Ltd. 

 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.

 3. Neither the name of AEP Ltd. nor the names of its contributors 
 may be used to endorse or promote products derived from this software 
 without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

 *****************************************************************************
****************************************************************************/

/**********************************************************************
*                                                                     *
* AEP Ltd.                                                            *
*                                                                     *
* This software is supplied under the terms of the license included   *
* above.  All use of this software must be in accordance with the     *
* terms of that license.                                              *
*                                                                     *
**********************************************************************/
/*This module contains gotos (towards end of file.)  May have to be removed for FIPS.*/

#include <time.h>
#include <errno.h>
#include <pthread.h>
#include <stdlib.h>
#include <string.h>

#include "aep_i_defs.h"
#include "aep_i_types.h"
#include "aep_daemon_defs.h"
#include "config.h"

#include "aepmalloc.h"

#include "mutex.h"

#include "hash.h"
#include "aeptrace.h"
#include "connection.h"
#include "aependian.h"

#include "ctx_hndlr.h"

/*Global data required by the ctx hndlr thread*/

extern AEP_U32 SABuffIndex;

/*This is where the requests are buffered while waiting to be sent to the accelerator*/
extern AEP_U64* ASIC_CurRequestBuffer;
extern AEP_U64* DES_CurRequestBuffer;
extern AEP_U64* SA_CurRequestBuffer;

extern pthread_mutex_t SA_RequestBuffer_mutex;

extern pthread_mutex_t response_item_mutex_array[];

extern pthread_mutex_t AEP_ContextID_mutex;
extern pthread_cond_t request_contexts;

extern AEP_U32 SendSABuffer;

/*Condition variables used to wait for the request buffers to be sent/to need sending*/
extern pthread_cond_t SA_send_now, SA_send_complete;

/*Semaphore used to hold off taking requests before the daemon authenticates
	and requests some contexts*/
extern sem_t auth_semaphore;

extern AEP_U64 contextID_array[NUM_CONTEXTS_REQUESTED + CONTEXTS_LOW_MARK + 1];

extern AEP_U32 contextID_count;

/*This variable holds the deamons context ID (App ID) that is sent with
every request for more contexts*/
extern AEP_U64 daemon_context_id;

extern AEP_U32 alive_message_flag;

/*This variable holds the daemons process id, which is used to make up tran id's*/
extern AEP_U32 proc_ID;


/*This thread handles daemon authentication and context requesting and the handling of the
response that comes back from the context request*/

void *ptrContextHndlr()
{
  /*Message header used to send requests to the daemon*/
	AEP_MSG_HDR_PTR 	message_hdr;
	AEP_U64 message[256 / 8];
/*	AEP_U8 daemon_name[20];*/
/*	int fd;*/
	
	AEP_U32		count;
	AEP_U64*	context_id_ptr;
	
  /*These two structs are used to calculate the timeout for the condition variable*/
	struct timespec timeout;
	struct timeval timenow; 
	
  /*Pointer to response hash table element*/
	AEP_I_RESPONSE_DATA_PTR pResponse;
	
	AEP_U32 tran_id, padding, hash_value, num_context_ids;
	
	AEP_RV rv;
		
  /*Generate a new transaction id for the daemon authentication message*/
  	rv = AEP_I_NewTranID(&tran_id);
	
	if (rv != AEP_R_OK)
	{
		AEP_TRACE (DAEMON, TRACE_ERRORS, FN_NONE, "ptrContextHndlr", rv, "Failed to get a Transaction ID for the Daemon's Authentication - Exiting...");
		exit(1);
	}
		
  /*Make up the authenticate daemon message*/
	message_hdr = (AEP_MSG_HDR_PTR) &message[0];
	message_hdr->app_id = 0;
	message_hdr->trans_hndl = (AEP_U64)proc_ID << 32 | tran_id;
	message_hdr->command = AEP_C_AUTH_DAEMON;
	message_hdr->flags = AEP_F_SA;
	
  /*Make up the message data, just the daemon name*/
	strcpy((AEP_CHAR*) message + HDR_DATA_OFFSET, "AEP Daemon Proc");
	message_hdr->datalen = (AEP_U32) strlen((AEP_CHAR*) message + HDR_SIZE) + 1;
	
  /*See if we need to add any padding*/
	padding = message_hdr->datalen & PADDING_MASK;

  /*If padding is not 0 then we need to add some*/
	if (padding)
	  /*The amount needed is 8 - the value of padding.  i.e. if padding was 1 then
		the datalen must be just 1 byte over a quad word boundry, so we must add
		7 bytes of padding (8 - 1).*/
		padding = sizeof(AEP_U64) - padding;
		
  /*Get the hash value for this transaction*/
	hash_value = AEP_I_HASH(message_hdr->trans_hndl);
	
	_LOCK(&SA_RequestBuffer_mutex);
			
   /*If adding this request to the request buffer will overflow the buffer, then
   send it for processing*/
	if (SABuffIndex + message_hdr->datalen + HDR_SIZE + padding >= SA_BufferSize)
	{
	  /*The buffer should be sent.  This bool is checked in the request buffer handler thread*/
		SendSABuffer = TRUE;

	  /*Signal the send buffer condition variable*/
		pthread_cond_signal(&SA_send_now);

	  /*The request buffer hndlr thread resets the buffer index back to 0 when finished sending,
	  effectively emptying the buffer.  So this comparision will fail when the thread is finished*/
		while(SABuffIndex + message_hdr->datalen + HDR_SIZE >= SA_BufferSize)/*BUFFER_SIZE)*/
		  /*Wait until the request buffer handler thread signals us*/
			pthread_cond_wait(&SA_send_complete, &SA_RequestBuffer_mutex);
	}
		
 	/*Copy the message header accross.  Note : Memcpy would not give the correct
	endianess over 64 bits*/
  /*Copy the request header into the request buffer*/
  	memcpy((AEP_U8*)SA_CurRequestBuffer + SABuffIndex, (AEP_U8*)message, HDR_SIZE);
	
  /*Adjust the endianess of the header if necessary*/
  	BEndToLEnd((AEP_U8*) SA_CurRequestBuffer + SABuffIndex, HDR_SIZE /
		sizeof(AEP_U32));

	/*Copy the message data accross*/
	memcpy((AEP_U8*)SA_CurRequestBuffer + HDR_SIZE, 
		(AEP_U8*)message + HDR_SIZE, message_hdr->datalen);
	
	SABuffIndex += (message_hdr->datalen + HDR_SIZE + padding);
	
  /*allocate some memory for the entry that will go into the response table for this tran*/
	pResponse = (AEP_I_RESPONSE_DATA_PTR) AEP_malloc(sizeof(AEP_I_RESPONSE_DATA));
	if(pResponse == NULL_PTR)
	{
		AEP_TRACE (DAEMON, TRACE_ERRORS, FN_NONE, "ptrContextHndlr", AEP_R_HOST_MEMORY, "Error : Host Memory Allocation Failure - Exiting...");
		exit(1);
	}

  /*Prepare to add a request header to the responses hash table*/
	pResponse->transaction_handle = message_hdr->trans_hndl;

  /*Get some space to hold the message hdr in the response table*/
	pResponse->msg = (AEP_U64_PTR) AEP_malloc(sizeof(AEP_MSG_HDR));
	
  /*Copy in the message that was just made up (We can't just use the pointer to the
    message because we may need to change it's endianess*/
	memcpy((AEP_U8*)pResponse->msg, message_hdr, sizeof(AEP_MSG_HDR));

  /*Set up the response table item's time stamp*/
	gettimeofday(&pResponse->timestamp, NULL);

  /*Add the timeout period*/
	pResponse->timestamp.tv_sec += Request_Timeout;
	
/*Lock the response table mutex*/
	_LOCK(&response_item_mutex_array[hash_value]);

  /*Insert the response entry into the hash table*/
	rv = AEP_I_HashPutEntry(AEP_I_response_table.response_table,pResponse->transaction_handle,(AEP_VOID_PTR)pResponse);
	if(rv != AEP_R_OK)
	{
		AEP_free(pResponse->msg);
		AEP_free(pResponse);
		_UNLOCK(&response_item_mutex_array[hash_value]);
		AEP_TRACE (DAEMON, TRACE_ERRORS, FN_NONE, "ptrContextHndlr", rv, "Error adding Response to Hash Table - Exiting...");
		exit(1); 
	}

	/*Get space for and initialize the semaphore that will be used to wait for the response*/
	pResponse->response_semaphore = (sem_t *) AEP_malloc(sizeof(sem_t));
	if (pResponse->response_semaphore == NULL)
	{
		_UNLOCK(&response_item_mutex_array[hash_value]);
		AEP_TRACE (DAEMON, TRACE_ERRORS, FN_NONE, "ptrContextHndlr", AEP_R_HOST_MEMORY, "Error : Host Memory Allocation Failure - Exiting...");
		exit(1);
	}

	SEM_INIT(pResponse->response_semaphore, 0, 0);

/*Unlock the request buffer mutex*/
	_UNLOCK(&SA_RequestBuffer_mutex);

/*UNLOCK the response table mutex*/
	_UNLOCK(&response_item_mutex_array[hash_value]);

  /*Wait for the semaphjore to be signalled indicating either response is back or time out*/
	SEM_WAIT(pResponse->response_semaphore);

/*Lock the response table mutex*/
	_LOCK(&response_item_mutex_array[hash_value]);

  /*Uninitialise and free the semaphore used to wait for the reply*/
	sem_destroy(pResponse->response_semaphore);
	AEP_free(pResponse->response_semaphore);

  /*Make this NULL again, ie no one waiting on semaphore*/
	pResponse->response_semaphore = NULL;
	
  /*If this request times out printf a message and exit*/
	if ((AEP_U64)((AEP_MSG_HDR_PTR)pResponse->msg)->status != AEP_S_COMPLETE)
	{
		
		AEP_TRACE (DAEMON, TRACE_ERRORS, FN_NONE, "ptrContextHndlr", AEP_R_TRANSACTION_NOT_READY, "Error: Response not yet ready - Exiting...");
		printf("Could not get daemon id from accelerator\n");
		exit(1);
	}
	  
	daemon_context_id = (AEP_U64)((AEP_MSG_HDR_PTR)pResponse->msg)->app_id;

  /*Remove the response entry from the hash table*/
	rv = AEP_I_HashRemoveEntry(AEP_I_response_table.response_table,pResponse->transaction_handle);
	if(rv != AEP_R_OK)
	{	
		AEP_free(pResponse->msg);
		AEP_free(pResponse);
		_UNLOCK(&response_item_mutex_array[hash_value]);
		AEP_TRACE (DAEMON, TRACE_ERRORS, FN_NONE, "ptrContextHndlr", rv, "Removing Response from Hash Table");
		exit(1); 
	}

  /*Free the message part of the response item*/
	AEP_free(pResponse->msg);

  /*Free the mem used to hold the response entry*/
	AEP_free(pResponse);
	
	_UNLOCK(&response_item_mutex_array[hash_value]);
	
  /*Post the semaphore allowing the connection handler to continue*/
	sem_post(&auth_semaphore);
			
  /*Infinitely loop waiting on the request contexts condition variable*/
	while(1)
	{
	  /*Initialise the alive flag and the rv value*/
		alive_message_flag = 1;
		rv=0;
		
		_LOCK(&AEP_ContextID_mutex);
		
		/*While there has been something sent to the target in the last timeout period
			and we have sufficient contexts*/
		while (alive_message_flag == 1 && contextID_count > CONTEXTS_LOW_MARK)
		{
		  /*If we have returned because of a timeout, if this is the first time, rv will be 0*/
			if (rv == ETIMEDOUT)
				/*We woke on the condition variable because we timed out.  Mark the 
				alive msg flag 0 and go asleep again.  If we time out again we'll send an
				alive message*/
				alive_message_flag = 0;
					
		  /*When specifying how long the timeout period for the condition variable is
			an absolute time must be used.  So we must get the current time and add the
			timeout interval to that time*/
			gettimeofday(&timenow, NULL);
			timeout.tv_sec = timenow.tv_sec + 20;
			timeout.tv_nsec = timenow.tv_usec * 1000;

			/*Wait for the condition variable to be signalled or time out
				Note: This automatically releases the mutex.  On return we automatically
				reacquire the mutex*/
			rv = pthread_cond_timedwait(&request_contexts, &AEP_ContextID_mutex, &timeout);
		}
			
	  /*Generate a new transaction id for the message*/
	  	rv = AEP_I_NewTranID(&tran_id);
		
		if (rv != AEP_R_OK)
		{
			AEP_TRACE (DAEMON, TRACE_ERRORS, FN_NONE, "ptrContextHndlr", rv, "Error : Failed to get a Transaction ID for the Daemon's Authentication");

			continue;	
		}
		/*Request more contexts*/
		if (contextID_count < CONTEXTS_LOW_MARK)
		{
			printf("request more ctxs\n");
			_UNLOCK(&AEP_ContextID_mutex);
		
		  /*Make up the request ctxs message*/
			message_hdr = (AEP_MSG_HDR_PTR) &message[0];
			message_hdr->app_id = daemon_context_id;
			message_hdr->trans_hndl = (AEP_U64)proc_ID << 32 | tran_id;
			message_hdr->command = AEP_C_REQUEST_CTXS;
			message_hdr->flags = AEP_F_SA;
			
		  /*Set the datalen to be the amount that should be returned*/
			message_hdr->datalen = NUM_CONTEXTS_REQUESTED * sizeof(AEP_U64);
			
  		  /*Set the data equal to the number of required contexts*/
			*((AEP_U32_PTR)(message + (HDR_SIZE/8))) = NUM_CONTEXTS_REQUESTED;
		  
		  /*See if we need to add any padding*/
			padding = message_hdr->datalen & PADDING_MASK;

 		 /*If padding is not 0 then we need to add some*/
			if (padding)
			  /*The amount needed is 8 - the value of padding.  i.e. if padding was 1 then
				the datalen must be just 1 byte over a quad word boundry, so we must add
				7 bytes of padding (8 - 1).*/
				padding = sizeof(AEP_U64) - padding;
				
	  	  /*Get the hash value for this transaction*/
			hash_value = AEP_I_HASH(message_hdr->trans_hndl);
				
/*Lock the SA request buffer mutex*/	
			_LOCK(&SA_RequestBuffer_mutex);
				
	     /*If adding this request to the request buffer will overflow the buffer, then
   			send it for processing*/
			if (SABuffIndex + message_hdr->datalen + HDR_SIZE + padding >= SA_BufferSize)
			{
	   		  /*The buffer should be sent.  This bool is checked in the request buffer handler thread*/
				SendSABuffer = TRUE;

			  /*Signal the send buffer condition variable*/
				pthread_cond_signal(&SA_send_now);

	 		 /*The request buffer hndlr thread resets the buffer index back to 0 when finished sending,
			  effectively emptying the buffer.  So this comparision will fail when the thread is finished*/
				while(SABuffIndex + message_hdr->datalen + HDR_SIZE + padding >= SA_BufferSize)/*BUFFER_SIZE)*/
			  /*Wait until the request buffer handler thread signals us*/
					pthread_cond_wait(&SA_send_complete, &SA_RequestBuffer_mutex);
			}
			
 		  /*Copy the request header into the request buffer*/
 		 	memcpy((AEP_U8*)SA_CurRequestBuffer + SABuffIndex, (AEP_U8*)message, HDR_SIZE);
	
		  /*Copy the message data accross*/
			memcpy((AEP_U8*)SA_CurRequestBuffer + SABuffIndex + HDR_SIZE, 
				(AEP_U8*)message + HDR_SIZE, sizeof(AEP_U32));
			
			/*Adjust the endianess of the header and request ctx data if necessary*/
  			BEndToLEnd((AEP_U8*)SA_CurRequestBuffer + SABuffIndex, (HDR_SIZE /
				sizeof(AEP_U32)) + 1);
		
			SABuffIndex += (message_hdr->datalen + HDR_SIZE + padding);
			
  		  /*allocate some memory for the entry that will go into the response table for this tran*/
			pResponse = (AEP_I_RESPONSE_DATA_PTR) AEP_malloc(sizeof(AEP_I_RESPONSE_DATA));
			if(pResponse == NULL_PTR)
			{
				AEP_TRACE (DAEMON, TRACE_ERRORS, FN_NONE, "ptrContextHndlr", AEP_R_HOST_MEMORY, "Error: Host Memory Allocation Failure");
				continue;
			}

 		  /*Prepare to add a request header to the responses hash table*/
			pResponse->transaction_handle = message_hdr->trans_hndl;

  		  /*Get some space to hold the message hdr in the response table*/
			pResponse->msg = (AEP_U64_PTR) AEP_malloc(sizeof(AEP_MSG_HDR));
			if(pResponse->msg == NULL_PTR)
			{
				AEP_TRACE (DAEMON, TRACE_ERRORS, FN_NONE, "ptrContextHndlr", AEP_R_HOST_MEMORY, "Error : Host Memory Allocation Failure");
				
				continue;
			}
	
  		  /*Copy in the message that was just made up (We can't just use the pointer to the
  		  message because we need to change it's endianess*/
			memcpy((AEP_U8*)pResponse->msg, message_hdr, sizeof(AEP_MSG_HDR));

  		  /*Set up the response table item's time stamp*/
			gettimeofday(&pResponse->timestamp, NULL);

	      /*Add the timeout period*/
			pResponse->timestamp.tv_sec += Request_Timeout;
	
		  /*Lock the response table mutex*/
			_LOCK(&response_item_mutex_array[hash_value]);

  		  /*Insert the response entry into the hash table*/
			rv = AEP_I_HashPutEntry(AEP_I_response_table.response_table,pResponse->transaction_handle,(AEP_VOID_PTR)pResponse);
			if(rv != AEP_R_OK)
			{
				AEP_free(pResponse->msg);
				AEP_free(pResponse);
				_UNLOCK(&response_item_mutex_array[hash_value]);
				AEP_TRACE (DAEMON, TRACE_ERRORS, FN_NONE, "ptrContextHndlr", rv, "Error adding Response to Hash Table");
				

				continue; 
			}

		  /*Get space for and initialize the semaphore that will be used to wait for the response*/
			pResponse->response_semaphore = (sem_t *) AEP_malloc(sizeof(sem_t));
			if (pResponse->response_semaphore == NULL)
			{
				AEP_free(pResponse->msg);
				AEP_free(pResponse);
				_UNLOCK(&response_item_mutex_array[hash_value]);
				AEP_TRACE (DAEMON, TRACE_ERRORS, FN_NONE, "ptrContextHndlr", AEP_R_HOST_MEMORY, "Error: Host Memory Allocation Failure");

				continue;
			}

			SEM_INIT(pResponse->response_semaphore, 0, 0);
			
		  /*Unlock the request buffer mutex*/
			_UNLOCK(&SA_RequestBuffer_mutex);

	      /*UNLOCK the response table mutex*/
			_UNLOCK(&response_item_mutex_array[hash_value]);

	      /*Wait for the semaphjore to be signalled indicating either response is back or time out*/
			SEM_WAIT(pResponse->response_semaphore);
	
			_LOCK(&response_item_mutex_array[hash_value]);

			sem_destroy(pResponse->response_semaphore);
			AEP_free(pResponse->response_semaphore);

	      /*Make this NULL again, ie no one waiting on semaphore*/
			pResponse->response_semaphore = NULL;
	
	      /*If this request times out printf a message and exit*/
			if ((AEP_U64)((AEP_MSG_HDR_PTR)pResponse->msg)->status != AEP_S_COMPLETE)
			{
				_UNLOCK(&response_item_mutex_array[hash_value]);
				AEP_TRACE (DAEMON, TRACE_ERRORS, FN_NONE, "ptrContextHndlr", AEP_R_TRANSACTION_NOT_READY, "Accelerator returned no Contexts");
				
				continue;
			}

/*Lock the context id mutex*/
			_LOCK(&AEP_ContextID_mutex);
			
			/*Here is where we do the stuff with the data that came back!*/
			/*Work out how many contexts were returned*/
			num_context_ids = ((AEP_U32)((AEP_MSG_HDR_PTR)pResponse->msg)->datalen) / sizeof(AEP_U64);

			/*Point to the first context*/
			context_id_ptr = (AEP_U64*) (pResponse->msg + (HDR_SIZE / 8));
					
			/*Stick each context in the context array*/	
			for (count = 0; count < num_context_ids; count++)
			{
				contextID_array[contextID_count++] = *context_id_ptr;
				
				/*Go to the next context in the reply*/
				context_id_ptr++;
			}

/*UnLock the context id mutex*/		
			_UNLOCK(&AEP_ContextID_mutex);
			
		  /*Remove the response entry from the hash table*/
			rv = AEP_I_HashRemoveEntry(AEP_I_response_table.response_table,pResponse->transaction_handle);
			if(rv != AEP_R_OK)
			{	
				AEP_free(pResponse->msg);
				AEP_free(pResponse);
				_UNLOCK(&response_item_mutex_array[hash_value]);
				AEP_TRACE (DAEMON, TRACE_ERRORS, FN_NONE, "ptrContextHndlr", rv, "Error removing Response from Hash Table");

				continue; 
			}

		  /*Free the message part of the response item*/
			AEP_free(pResponse->msg);

		  /*Free the mem used to hold the response entry*/
			AEP_free(pResponse);
	
			_UNLOCK(&response_item_mutex_array[hash_value]);
		}
		else
		/*Send an alive message*/
		{
			/*No longer need to keep this mutex*/
			_UNLOCK(&AEP_ContextID_mutex);
			
	/*printf("Alive\n");*/
			
		  /*Make up the alive message*/
			message_hdr = (AEP_MSG_HDR_PTR) &message[0];
			message_hdr->app_id = daemon_context_id;
			message_hdr->trans_hndl = (AEP_U64)proc_ID << 32 | tran_id;
			message_hdr->command = AEP_C_ALIVE;
			message_hdr->flags = AEP_F_SA | AEP_F_CTXCMD;	
			message_hdr->datalen = 0;

/*Lock the SA request buffer mutex*/	
			_LOCK(&SA_RequestBuffer_mutex);
				
	     /*If adding this request to the request buffer will overflow the buffer, then
   			send it for processing*/
			if (SABuffIndex + HDR_SIZE >= SA_BufferSize)
			{
	   		  /*The buffer should be sent.  This bool is checked in the request buffer handler thread*/
				SendSABuffer = TRUE;

			  /*Signal the send buffer condition variable*/
				pthread_cond_signal(&SA_send_now);

	 		 /*The request buffer hndlr thread resets the buffer index back to 0 when finished sending,
			  effectively emptying the buffer.  So this comparision will fail when the thread is finished*/
				while(SABuffIndex + HDR_SIZE >= SA_BufferSize)/*BUFFER_SIZE)*/
			  /*Wait until the request buffer handler thread signals us*/
					pthread_cond_wait(&SA_send_complete, &SA_RequestBuffer_mutex);
			}
		  
 		 /*Copy the request header into the request buffer*/
 		 	memcpy((AEP_U8*)SA_CurRequestBuffer + SABuffIndex, (AEP_U8*)message_hdr, HDR_SIZE);
	
		  /*Adjust the endianess of the header if necessary*/
 		 	BEndToLEnd((AEP_U8*)SA_CurRequestBuffer + SABuffIndex, HDR_SIZE /
				sizeof(AEP_U32));
		  /*Incrememnt the SA Buffer index by the size of an alive message*/
			SABuffIndex += HDR_SIZE;
	
/*UnLock the SA request buffer mutex*/	
			_UNLOCK(&SA_RequestBuffer_mutex);
		}
	}
}
