/*****************************************************************************
 *****************************************************************************
 Copyright (c) 2000-2001, AEP Ltd. 

 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.

 3. Neither the name of AEP Ltd. nor the names of its contributors 
 may be used to endorse or promote products derived from this software 
 without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

 *****************************************************************************
****************************************************************************/

/**********************************************************************
*                                                                     *
* AEP Ltd.                                                            *
*                                                                     *
* This software is supplied under the terms of the license included   *
* above.  All use of this software must be in accordance with the     *
* terms of that license.                                              *
*                                                                     *
**********************************************************************/
/*	ptrResponseHndlr
	This function defines the thread that handles the reading and parsing of the response buffer

*/

#include <stdlib.h>
#include <pthread.h>
#include <string.h>
#include <stdio.h>

#include <unistd.h>

#include "aep_daemon_defs.h"
#include "aep_i_types.h"
#include "aep_i_defs.h"
#include "config.h"

#include "aepmalloc.h"
#include "mutex.h"
#include "hash.h"
#include "sockets.h"
#include "aependian.h"
#include "aeptrace.h"

#include "response_hndlr.h"

extern pthread_mutex_t response_item_mutex_array[];
extern pthread_mutex_t AEP_KeyMan_mutex;

extern sem_t read_done_array_semaphore;

extern pthread_cond_t ASIC_BufferFull_cond;
extern pthread_cond_t DES_BufferFull_cond;
extern pthread_cond_t SA_BufferFull_cond;

/*File manager socket descriptor*/
extern int keydb_fd;

extern int AccFd;




void *ptrResponseHndlr()
{
	AEP_I_RESPONSE_DATA_PTR pResponse;
	AEP_MSG_HDR_PTR		response_header;
	
	AEP_U32				datalen, flags;
	AEP_U32				AccComsError = 0;
	
	AEP_U64				response_buffer[MAX_READ_BUFFER_SIZE/sizeof(AEP_U64)];	

	int bytes_read, index, hash_value;

	AEP_U32 rv, padding;

	
 /*Check the request buffer for ever*/
	while(1)
	{
		bytes_read = index = 0;
		
	/*	printf("before read\n");*/
		
		/*Try to read a block of responses*/
		bytes_read = read(AccFd, (AEP_U8*)response_buffer, ReadBufferSize);
		
	/*	printf("Bytes read %d\n", bytes_read);*/
		
		/*Check for read error.  If we get 10 in a row we can be fairly sure
			that there's something serious wrong so exit*/
		while (bytes_read == -1)
		{
			/* printf("bad r\n"); */
			AccComsError++;
			
			if (AccComsError == 10)
			{
				AEP_TRACE (DAEMON, TRACE_ERRORS, FN_NONE, "ptrResponseHndlr", AEP_R_TARGET_ERROR, "Error : Accelerator Communications Error - Exiting...");
				perror("Accelerator comms\n");
				exit(1);
			}
			
			bytes_read = read(AccFd, (AEP_U8*)response_buffer, ReadBufferSize);
		}
						
		/*Reset the read error count*/
		AccComsError = 0;
		
	  /*While there are messages in the done array*/
		while(index < bytes_read)
		{
		/*	printf("got reply\n");*/
		  /*Parse a message from the response buffer.*/
			response_header = (AEP_MSG_HDR_PTR)(response_buffer + (index >> 3));
			
			LEndToBEnd((AEP_U8*)response_header, HDR_SIZE / sizeof(AEP_U32));
			
			hash_value = AEP_I_HASH(response_header->trans_hndl);
			
			flags = response_header->flags;
			
			datalen = response_header->datalen;
			
		  /*See if this response is to go to the key manager*/
		  	if(flags & AEP_F_RESTOKEYMAN)
			{
			  /*Is the key manager listening?*/
				if(keydb_fd != -1)
				{
					printf("resp going to key man\n");
				  /*Allocate enough memory for the response*/	
					
/*Lock the key man mutex.*/
					_LOCK(&AEP_KeyMan_mutex);
	
				  /*Write the request to the Key Management application*/
				  	rv = AEP_I_SendToSocket(keydb_fd, (AEP_U8*)response_header, HDR_SIZE +
						datalen);

/*Unlock the keyman mutex*/
					_UNLOCK(&AEP_KeyMan_mutex);

				  /*Check that the message was successfully sent to the key
					  manager*/
					if (rv != AEP_R_OK)
					  /*Key manager is no longer listening*/
						keydb_fd = -1;
				}
				else	printf("oh oh key man cmd and no key man\n");
			}
		  /*This response is to go back to the calling application*/
			else
			{
/*Lock the response table*/
				_LOCK(&response_item_mutex_array[hash_value]);
				
			  /*Find the response item in the response hash table*/	
				rv = AEP_I_HashGetEntry(AEP_I_response_table.response_table, response_header->trans_hndl, 
						(AEP_VOID_PTR) &pResponse);
							
			  /*Check to see if the response item exists*/
				if (rv != AEP_R_OK) 
				{		
					AEP_TRACE(DAEMON, TRACE_ERRORS, FN_NONE, "AEP_Daemon", 
						rv, "No Matching Request in the Hash Table.");
					/*Response item does not exist, so in the words of Donnie Brasco, "Forget about it"*/			  
				}
				else
				{
				  /*Copy the complete response into the response table*/

				  /*Free the existing msg header*/
					AEP_free(pResponse->msg);

				  /*If there's a valid semaphore (ie someones waiting on this transaction)
					then signal them to let them know the answer has arrived*/
					if(pResponse->response_semaphore != NULL)
					{
						AEP_I_SendToSocket(pResponse->sock_fd, (AEP_U8*)response_buffer + index,
							datalen + HDR_SIZE);
							
					  /*Remove the response entry from the hash table*/
						AEP_I_HashRemoveEntry(AEP_I_response_table.response_table,pResponse->transaction_handle);
						
						sem_post(pResponse->response_semaphore);									
					}
					else
					{
					  /*Allocate enough memory for the response*/	
						pResponse->msg = (AEP_U64_PTR) AEP_malloc(HDR_SIZE + datalen);
						if (pResponse->msg == NULL)
						{
							/* printf("returned null trying to alloc %d bytes", HDR_SIZE + response_header->datalen); */
							AEP_TRACE (DAEMON, TRACE_ERRORS, FN_NONE, "ptrResponseHndlr", AEP_R_HOST_MEMORY, "Error - Host Memory Allocation Failure");
						}
						else
						{
				  			/*Copy the message into the response item*/
					 		memcpy((AEP_U8*)pResponse->msg, (AEP_U8*)response_buffer + index,
					 			HDR_SIZE + datalen);
						}
					}
				}
/*Unlock the response table mutex*/
				_UNLOCK(&response_item_mutex_array[hash_value]);
			}

		  /*Determine if any padding is required*/
			padding = datalen & PADDING_MASK;

		  /*If padding is not 0 then we need to add some*/
			if (padding)
			{
			  /*The amount needed is 8 - the value of padding.  i.e. if padding was 1 then
				the datalen must be just 1 byte over a quad word boundry, so we must add
				7 bytes of padding (8 - 1).*/
				padding = sizeof(AEP_U64) - padding;
			}
		
			index += (HDR_SIZE + datalen + padding);
			
		} /*End 1st while*/

		/*Check if this was the last message in a block*/
/*		if (flags & AEP_F_LASTMSG)
		{
			switch (flags & AEP_F_LUNMASK)
			{
			case AEP_F_ASIC :*/
			  /*Signal the buffer full condition variable in case there's any threads
			  	waiting to try to write their buffers again*/
/*				pthread_cond_signal(&ASIC_BufferFull_cond);
				break;
			
			case AEP_F_DES :*/
			  /*Signal the buffer full condition variable in case there's any threads
			  	waiting to try to write their buffers again*/
/*				pthread_cond_signal(&DES_BufferFull_cond);
				break;
				
			case AEP_F_SA :*/
			  /*Signal the buffer full condition variable in case there's any threads
			  	waiting to try to write their buffers again*/
/*				pthread_cond_signal(&SA_BufferFull_cond);
				break;
			}			
		}*/
		
	} /*End infinite while*/
	
}
