/*****************************************************************************
 *****************************************************************************
 Copyright (c) 2000-2001, AEP Ltd. 

 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.

 3. Neither the name of AEP Ltd. nor the names of its contributors 
 may be used to endorse or promote products derived from this software 
 without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

 *****************************************************************************
****************************************************************************/

/**********************************************************************
*                                                                     *
* AEP Ltd.                                                            *
*                                                                     *
* This software is supplied under the terms of the license included   *
* above.  All use of this software must be in accordance with the     *
* terms of that license.                                              *
*                                                                     *
**********************************************************************/


/* **************************************************************/
/*								*/
/* Filename:	sockets.c					*/
/* Description:	This file contains the implementation of the 	*/
/*		AEP sockets functions. It really only wraps the */
/*		OS specific stuff to reduce the amount of #ifs	*/
/*		in the other modules.				*/
/*								*/
/* Copyright:	Accelerated Encryption Processing Ltd., 2000	*/
/* Author:	Diarmuid O'Neill				*/
/* Product:	AEP1000						*/
/* Date:	20/10/99					*/
/*								*/
/****************************************************************/

#include <sys/types.h>
#include <sys/un.h> 
#include <sys/stat.h>
#include <sys/socket.h>
#include <netdb.h>
#include <time.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h> /* exlicitly included here for linux */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>

#include "aep_i_defs.h"
#include "aep_i_types.h"
#include "sockets.h"
#include "aeptrace.h"

#define BACKLOG 50    /* how many pending connections queue will hold */

AEP_RV AEP_I_DaemonConnect(int* sock_fd)
{

#if defined(AEP_GENERIC)
	/*struct hostent *he;*/
  	struct sockaddr_un their_addr; /* connector's address information */

#else
	char buf[30];
#endif

	struct timespec interval;
	AEP_U32 connected;
	AEP_U32 connect_attempts = 0;

	AEP_RV rv = AEP_R_OK;
	char sockname[80];

	sprintf(sockname, AEP_DAEMON_SOCKET);

#if defined(AEP_Win32)
  /*Do the temporary file stuff!!!!!!REMOVE THIS!!!!!*/
	sprintf(buf,"Outputfile1.txt");

  /*Store the file descriptor Note: This will be a socket descriptor*/
	*sock_fd = _open(buf,_O_CREAT | _O_RDWR | _O_BINARY);

#elif defined(AEP_GENERIC)
	
/*	if ((he=gethostbyname("127.0.0.1")) == NULL) 
    	{  
      		perror("gethostbyname");
 			rv = AEP_R_GENERAL_ERROR;
			return rv; 
    	}*/
    	
	their_addr.sun_family = AF_UNIX;      /* host byte order */
	strcpy(their_addr.sun_path, sockname);
	
	/* When connecting if at first we do not succeed try and try again
		How many times we try and the interval between the tries is 
		determined by CONNECT_INTERVAL and CONNECT_RETRIES.  The timeout
		period can be determined by INTERVAL * RETRIES*/
	connected = FALSE;
	do
	{
	  /*Try to create a new socket*/
   		if ((*sock_fd = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) 
    	{
      		perror("socket");
			rv = AEP_R_GENERAL_ERROR;
			return rv; 

		}		
		
	  /*Attempt to connect to the daemon*/
		if (connect(*sock_fd,(struct sockaddr *)&their_addr,sizeof(struct sockaddr_un)) != -1)
		{
	  /*Successfull connection so set connected to true and break out of the do-while loop*/
			connected = TRUE;
    		break;
		}

	  /*If we're still here we failed to connect so close the created socket*/
		close(*sock_fd);

	  /*Set up the interval for the nanosleep*/
		interval.tv_sec = 0;
		interval.tv_nsec = CONNECT_INTERVAL;
	  
	  /*Go to sleep for a while*/
		nanosleep(&interval, NULL);
	
		connect_attempts ++;

	} while (connect_attempts < CONNECT_RETRIES);
  /*Alak and Alas, we failed to connect.  Return an error*/
	if (connected == FALSE)
	{
			rv = AEP_R_GENERAL_ERROR;
			return rv; 
	}

  /*If we are here we have succeeded in connecting...nice!*/
	
#endif

	return rv;
}

AEP_RV AEP_I_DaemonDisconnect(int sock_fd)
{
	if (close(sock_fd) == 0)
		return AEP_R_OK;
	else
		return AEP_R_SOCKERROR;
}



AEP_RV AEP_I_KeyDBConnect(int* sock_fd, AEP_U32 port)
{

#if defined(AEP_GENERIC)
	struct hostent *he;
  	struct sockaddr_in their_addr; /* connector's address information */

#else
	char buf[30];
#endif

	struct timespec interval;
	AEP_U32 connected;
	AEP_U32 connect_attempts = 0;

	AEP_RV rv = AEP_R_OK;

#if defined(AEP_Win32)
  /*Do the temporary file stuff!!!!!!REMOVE THIS!!!!!*/
	sprintf(buf,"Outputfile1.txt");

  /*Store the file descriptor Note: This will be a socket descriptor*/
	*sock_fd = _open(buf,_O_CREAT | _O_RDWR | _O_BINARY);

#elif defined(AEP_GENERIC)
	
	if ((he=gethostbyname("127.0.0.1")) == NULL) 
    	{  /* get the host info */
      		perror("gethostbyname");
 			rv = AEP_R_GENERAL_ERROR;
			return rv; 
    	}
    	
	their_addr.sin_family = AF_INET;      /* host byte order */
	their_addr.sin_port = htons(port);
	their_addr.sin_addr = *((struct in_addr *)he->h_addr);
	bzero(&(their_addr.sin_zero), 8);
	
	/* When connecting if at first we do not succeed try and try again
		How many times we try and the interval between the tries is 
		determined by CONNECT_INTERVAL and CONNECT_RETRIES.  The timeout
		period can be determined by INTERVAL * RETRIES*/
	connected = FALSE;
	do
	{
	  /*Try to create a new socket*/
   		if ((*sock_fd = socket(AF_INET, SOCK_STREAM, 0)) == -1) 
    	{
      		perror("socket");
			rv = AEP_R_GENERAL_ERROR;
			return rv; 

		}		
		
	  /*Attempt to connect to the daemon*/
		if (connect(*sock_fd,(struct sockaddr *)&their_addr,sizeof(struct sockaddr)) != -1)
		{
	  /*Successfull connection so set connected to true and break out of the do-while loop*/
			connected = TRUE;
    		break;
		}

	  /*If we're still here we failed to connect so close the created socket*/
		close(*sock_fd);

	  /*Set up the interval for the nanosleep*/
		interval.tv_sec = 0;
		interval.tv_nsec = CONNECT_INTERVAL;
	  
	  /*Go to sleep for a while*/
		nanosleep(&interval, NULL);
	
		connect_attempts ++;

	} while (connect_attempts < 1);
  /*Alak and Alas, we failed to connect.  Return an error*/
	if (connected == FALSE)
	{
			rv = AEP_R_GENERAL_ERROR;
			return rv; 
	}

  /*If we are here we have succeeded in connecting...nice!*/
	
#endif

	return rv;
}

AEP_RV AEP_I_KeyDBDisconnect(int sock_fd)
{
	if (close(sock_fd) == 0)
		return AEP_R_OK;
	else
		return AEP_R_SOCKERROR;
}



AEP_RV AEP_I_SendToSocket(int sock_fd, AEP_U8_PTR buf, AEP_U32 Len)
{
	int BytesWritten;

	while(Len > 0)
	{
again:
		BytesWritten = write(sock_fd, buf, Len);
		if (BytesWritten < 0)
		{
			if(errno == EINTR)
				goto again;
			else
				return AEP_R_SOCKERROR;
		}
		else if (BytesWritten == 0)
			return AEP_R_SOCKEOF;

		Len -= BytesWritten;
		buf += BytesWritten;
	}
	
		return AEP_R_OK;
}

AEP_RV AEP_I_ReadFromSocket(int sock_fd, AEP_U8_PTR buf, AEP_U32 Len)
{
	int BytesRead;
	BytesRead = read(sock_fd, buf, Len);

	if (BytesRead < 0)	
		return AEP_R_SOCKERROR;
	else if (BytesRead == 0)
		return AEP_R_SOCKEOF;

	return AEP_R_OK;
}


AEP_RV AEP_I_DaemonListen(int* sock_fd)
{
	struct sockaddr_un my_addr;    /* my address information */

  /*Delete the file representing the unix domain socket if it already exists*/
	unlink(AEP_DAEMON_SOCKET);
	
  /*Create the socket that will await connections*/
	if ((*sock_fd = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) 
    {
		AEP_TRACE(API, TRACE_ERRORS, FN_NONE, "AEP_I_DaemonListen", AEP_R_SOCKERROR, "Error : Trying to create the daemon socket ");				
		return(AEP_R_SOCKERROR);
    }

	my_addr.sun_family = AF_UNIX;         /* host byte order */
	strcpy(my_addr.sun_path, AEP_DAEMON_SOCKET);
 
  /*Bind to the created socket*/
	if (bind(*sock_fd, (struct sockaddr *)&my_addr, sizeof(struct sockaddr_un))== -1)
    {
		AEP_TRACE(API, TRACE_ERRORS, FN_NONE, "AEP_I_DaemonListen", AEP_R_SOCKERROR, "Error : Trying to bind the daemon socket ");				
		return(AEP_R_SOCKERROR);
    }
	
  /*Set up global access permission to the socket */
  /* NOTE: This is necessary as some OS's (namely Linux) only give the 
	socket root permissions by default. This hack will mend this  
	for the time being. Note also that this function will only be called 
	by the daemon itself, which WILL be running as root*/
	fchmod(*sock_fd, 0666);

  /*Begin listening for connections*/
	if (listen(*sock_fd, BACKLOG) == -1) 
    {
		AEP_TRACE(API, TRACE_ERRORS, FN_NONE, "AEP_I_DaemonListen", AEP_R_SOCKERROR, "Error : Trying to listen on the daemon socket ");					
		return(AEP_R_SOCKERROR);
    }
	return AEP_R_OK;
}

AEP_RV AEP_I_DaemonWait(int* sock_fd, int* new_fd)
{
	struct sockaddr_un their_addr; /* connector's address information */
	AEP_U32 sin_size;

	sin_size = sizeof(struct sockaddr_un);

again:
  /*Get the next connection from the queue or block until a connection arrives*/
    if ((*new_fd = accept(*sock_fd, (struct sockaddr *)&their_addr,(int *)&sin_size)) == -1) 
	{
		if (errno == EINTR)
			goto again;
		else
		{
			AEP_TRACE(API, TRACE_ERRORS, FN_NONE, "AEP_I_DaemonWait", AEP_R_SOCKERROR, "Error : Accepting a new connection");			
			return(AEP_R_SOCKERROR);
		}
	}

	return AEP_R_OK;
}
